package app.k9mail.core.ui.compose.theme2.default

import androidx.compose.ui.unit.dp
import app.k9mail.core.ui.compose.theme2.ThemeSpacings

val defaultThemeSpacings = ThemeSpacings(
    zero = 0.dp,
    quarter = 2.dp,
    half = 4.dp,
    default = 8.dp,
    oneHalf = 12.dp,
    double = 16.dp,
    triple = 24.dp,
    quadruple = 32.dp,
)
