package com.fsck.k9.mailstore

object MessageColumns {
    const val ID = "id"
    const val UID = "uid"
    const val INTERNAL_DATE = "internal_date"
    const val SUBJECT = "subject"
    const val DATE = "date"
    const val MESSAGE_ID = "message_id"
    const val SENDER_LIST = "sender_list"
    const val TO_LIST = "to_list"
    const val CC_LIST = "cc_list"
    const val BCC_LIST = "bcc_list"
    const val REPLY_TO_LIST = "reply_to_list"
    const val FLAGS = "flags"
    const val ATTACHMENT_COUNT = "attachment_count"
    const val FOLDER_ID = "folder_id"
    const val PREVIEW_TYPE = "preview_type"
    const val PREVIEW = "preview"
    const val READ = "read"
    const val FLAGGED = "flagged"
    const val ANSWERED = "answered"
    const val FORWARDED = "forwarded"
}
